/*
 * aTunes 1.13.6
 * Copyright (C) 2006-2009 Alex Aranda, Sylvain Gaudard, Thomas Beckers and contributors
 *
 * See http://www.atunes.org/wiki/index.php?title=Contributing for information about contributors
 *
 * http://www.atunes.org
 * http://sourceforge.net/projects/atunes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package net.sourceforge.atunes.gui.images;

import java.net.URL;

import javax.swing.ImageIcon;

import net.sourceforge.atunes.misc.SystemProperties;

/**
 * The Class ImageLoader.
 * 
 * @author fleax
 */
public class ImageLoader {

    // Every time a image is added, an attribute must be added
    /** The Constant ADD. */
    public static final ImageIcon ADD = getImage("add.png");

    /** The Constant RADIO_ADD. */
    public static final ImageIcon RADIO_ADD = getImage("radio-add.png");

    /** The Constant ALBUM. */
    public static final ImageIcon ALBUM = getImage("album.png");

    /** The Constant ALBUM_FAVORITE. */
    public static final ImageIcon ALBUM_FAVORITE = getImage("albumFavorite.png");

    /** The Constant APP_TITLE. */
    public static final ImageIcon APP_TITLE = getImage("title.png");

    /** The Constant APP_ICON. */
    public static final ImageIcon APP_ICON = getImage("appIcon.png");

    /** The Constant APP_ICON_TRAY. */
    public static final ImageIcon APP_ICON_TRAY = getImage("appIconTray.png");

    public static final ImageIcon APP_ICON_MEDIUM = getImage("appIconMedium.png");

    /** The Constant APP_ICON_BIG. */
    public static final ImageIcon APP_ICON_BIG = getImage("appIconBig.png");

    /** The Constant APP_ICON_TINY. */
    public static final ImageIcon APP_ICON_TINY = getImage("appIconTiny.png");

    /** The Constant APP_ICON_TITLE. */
    public static final ImageIcon APP_ICON_TITLE = getImage("appIconTitle.png");

    /** The Constant ARTIST. */
    public static final ImageIcon ARTIST = getImage("artist.png");

    /** The Constant ARTIST_FAVORITE. */
    public static final ImageIcon ARTIST_FAVORITE = getImage("artistFavorite.png");

    /** The Constant ARTIST_SIMILAR. */
    public static final ImageIcon ARTIST_SIMILAR = getImage("artistSimilar.png");

    /** The Constant AUDIO_FILE. */
    public static final ImageIcon AUDIO_FILE = getImage("audioFile.png");

    /** The Constant AUDIO_FILE_LITTLE. */
    public static final ImageIcon AUDIO_FILE_LITTLE = getImage("audioFile_little.png");

    /** The Constant BALANCE_LEFT. */
    public static final ImageIcon BALANCE_LEFT = getImage("balance-left.png");

    /** The Constant BALANCE_MED_LEFT. */
    public static final ImageIcon BALANCE_MED_LEFT = getImage("balance-med-left.png");

    /** The Constant BALANCE_MED. */
    public static final ImageIcon BALANCE_MED = getImage("balance-med.png");

    /** The Constant BALANCE_MED_RIGHT. */
    public static final ImageIcon BALANCE_MED_RIGHT = getImage("balance-med-right.png");

    /** The Constant BALANCE_RIGHT. */
    public static final ImageIcon BALANCE_RIGHT = getImage("balance-right.png");

    /** The Constant CANCEL. */
    public static final ImageIcon CANCEL = getImage("cancel.png");

    /** The Constant CD_AUDIO. */
    public static final ImageIcon CD_AUDIO = getImage("cdAudio.png");

    /** The Constant CD_AUDIO_TINY. */
    public static final ImageIcon CD_AUDIO_TINY = getImage("cdAudioTiny.png");

    /** The Constant CD_COVER. */
    public static final ImageIcon CD_COVER = getImage("cd-cover.png");

    /** The Constant CHECK_FOR_UPDATES. */
    public static final ImageIcon CHECK_FOR_UPDATES = getImage("checkForUpdates.png");

    /** The Constant CHECK_FOR_UPDATES_BW. */
    public static final ImageIcon CHECK_FOR_UPDATES_BW = getImage("checkForUpdatesBW.png");

    /** The Constant CLEAR. */
    public static final ImageIcon CLEAR = getImage("clear.png");

    /** The Constant CLOSE_BUTTON. */
    public static final ImageIcon CLOSE_BUTTON = getImage("closeButton.png");

    /** The Constant CLOSE_TAB. */
    public static final ImageIcon CLOSE_TAB = getImage("closeTab.png");

    /** The Constant COLLAPSE. */
    public static final ImageIcon COLLAPSE = getImage("collapse.png");

    /** The Constant DATE. */
    public static final ImageIcon DATE = getImage("date.png");

    /** The Constant DELETE_FILE. */
    public static final ImageIcon DELETE_FILE = getImage("delete.png");

    /** The Constant DELETE_TAG. */
    public static final ImageIcon DELETE_TAG = getImage("erase.png");

    /** The Constant DEVICE. */
    public static final ImageIcon DEVICE = getImage("device.png");

    /** The Constant DEVICE_CONNECT. */
    public static final ImageIcon DEVICE_CONNECT = getImage("deviceConnect.png");

    /** The Constant DEVICE_DECONNECT. */
    public static final ImageIcon DEVICE_DECONNECT = getImage("deviceDeconnect.png");

    /** The Constant DEVICE_REFRESH. */
    public static final ImageIcon DEVICE_REFRESH = getImage("deviceRefresh.png");

    /** The Constant DOWNLOAD_PODCAST. */
    public static final ImageIcon DOWNLOAD_PODCAST = getImage("rss_download.png");

    /** The Constant EDIT_ALBUM. */
    public static final ImageIcon EDIT_ALBUM = getImage("editAlbum.png");

    /** The Constant EMPTY. */
    public static final ImageIcon EMPTY = getImage("empty.png");

    /** The Constant EMPTY_CONTEXT. */
    public static final ImageIcon EMPTY_CONTEXT = getImage("emptyContext.png");

    /** The Constant EXIT. */
    public static final ImageIcon EXIT = getImage("exit.png");

    /** The Constant EXPAND. */
    public static final ImageIcon EXPAND = getImage("expand.png");

    /** The Constant EXPORT. */
    public static final ImageIcon EXPORT = getImage("export.png");

    /** The Constant EXPORT_FILE. */
    public static final ImageIcon EXPORT_FILE = getImage("export-files.png");

    /** The Constant EXPORT_PICTURE. */
    public static final ImageIcon EXPORT_PICTURE = getImage("exportPicture.png");

    /** The Constant FAVORITE. */
    public static final ImageIcon FAVORITE = getImage("favorite.png");

    /** The Constant FILE. */
    public static final ImageIcon FILE = getImage("file.png");

    /** The Constant FILE_NAME. */
    public static final ImageIcon FILE_NAME = getImage("file_name.png");

    /** The Constant FOLDER. */
    public static final ImageIcon FOLDER = getImage("repository.png");

    /** The Constant FOLDER_ORANGE. */
    public static final ImageIcon FOLDER_ORANGE = getImage("folder.png");

    /** The Constant FULLSCREEN. */
    public static final ImageIcon FULLSCREEN = getImage("window-fullscreen.png");

    /** The Constant GENRE. */
    public static final ImageIcon GENRE = getImage("genre.png");

    /** The Constant GO_BOTTOM. */
    public static final ImageIcon GO_BOTTOM = getImage("go-bottom.png");

    /** The Constant GO_DOWN. */
    public static final ImageIcon GO_DOWN = getImage("go-down.png");

    /** The Constant GO_TOP. */
    public static final ImageIcon GO_TOP = getImage("go-top.png");

    /** The Constant GO_UP. */
    public static final ImageIcon GO_UP = getImage("go-up.png");

    /** The Constant INFO. */
    public static final ImageIcon INFO = getImage("info.png");

    /** The Constant KARAOKE. */
    public static final ImageIcon KARAOKE = getImage("karaoke.png");

    /** The Constant LANGUAGE. */
    public static final ImageIcon LANGUAGE = getImage("language.png");

    /** The Constant LOG. */
    public static final ImageIcon LOG = getImage("loag.png");

    /** The Constant MARK_RSS_AS_READ. */
    public static final ImageIcon MARK_RSS_AS_READ = getImage("markAsRead.png");

    /** The Constant MINIMIZE_BUTTON. */
    public static final ImageIcon MINIMIZE_BUTTON = getImage("minimizeButton.png");

    /** The Constant NAVIGATE. */
    public static final ImageIcon NAVIGATE = getImage("navigate.png");

    /** The Constant NAVIGATION_TABLE. */
    public static final ImageIcon NAVIGATION_TABLE = getImage("navigationTable.png");

    /** The Constant NETWORK. */
    public static final ImageIcon NETWORK = getImage("network.png");

    /** The Constant NETWORK_LITTLE. */
    public static final ImageIcon NETWORK_LITTLE = getImage("network_little.png");

    /** The Constant NEW_PLAYLIST. */
    public static final ImageIcon NEW_PLAYLIST = getImage("new_playlist.png");

    /** The Constant NEW_PODCAST_ENTRY. */
    public static final ImageIcon NEW_PODCAST_ENTRY = getImage("new_podcast_entry.png");

    /** The Constant NEXT. */
    public static final ImageIcon NEXT = getImage("next.png");

    /** The Constant NEXT_TRAY. */
    public static final ImageIcon NEXT_TRAY = SystemProperties.OS.isVista() ? getImage("nextTrayVista.png") : getImage("nextTray.png");

    /** The Constant NEXT_TRAY_MENU. */
    public static final ImageIcon NEXT_TRAY_MENU = getImage("nextTrayMenu.png");

    /** The Constant NO_COVER. */
    public static final ImageIcon NO_COVER = getImage("noCover.gif");

    /** The Constant NO_COVER_AUDIOFILE_PROPERTIES. */
    public static final ImageIcon NO_COVER_AUDIOFILE_PROPERTIES = getImage("noCover_AudioFileProperties.png");

    /** The Constant NORMALIZATION. */
    public static final ImageIcon NORMALIZATION = getImage("normalization.png");

    /** The Constant NUMBER. */
    public static final ImageIcon NUMBER = getImage("tag_number.png");

    /** The Constant OSD. */
    public static final ImageIcon OSD = getImage("osd.png");

    /** The Constant PAUSE. */
    public static final ImageIcon PAUSE = getImage("pause.png");

    /** The Constant PAUSE_TINY. */
    public static final ImageIcon PAUSE_TINY = getImage("pauseTiny.png");

    /** The Constant PAUSE_TRAY. */
    public static final ImageIcon PAUSE_TRAY = SystemProperties.OS.isVista() ? getImage("pauseTrayVista.png") : getImage("pauseTray.png");

    /** The Constant PAUSE_TRAY_MENU. */
    public static final ImageIcon PAUSE_TRAY_MENU = getImage("pauseTrayMenu.png");

    /** The Constant PLAY. */
    public static final ImageIcon PLAY = getImage("play.png");

    /** The Constant PLAY_LIST_CONTROLS. */
    public static final ImageIcon PLAY_LIST_CONTROLS = getImage("playListControls.png");

    /** The Constant PLAY_TINY. */
    public static final ImageIcon PLAY_TINY = getImage("playTiny.png");

    /** The Constant PLAY_MENU. */
    public static final ImageIcon PLAY_MENU = getImage("playMenu.png");

    /** The Constant PLAY_TRAY. */
    public static final ImageIcon PLAY_TRAY = SystemProperties.OS.isVista() ? getImage("playTrayVista.png") : getImage("playTray.png");

    /** The Constant PLAY_TRAY_MENU. */
    public static final ImageIcon PLAY_TRAY_MENU = getImage("playTrayMenu.png");

    /** The Constant PLAYLIST. */
    public static final ImageIcon PLAYLIST = getImage("playlist.png");

    /** The Constant POWERED_BY_LAST_FM. */
    public static final ImageIcon POWERED_BY_LAST_FM = getImage("poweredByLastFm.png");

    /** The Constant PREFS. */
    public static final ImageIcon PREFS = getImage("prefs.png");

    /** The Constant PREVIOUS. */
    public static final ImageIcon PREVIOUS = getImage("previous.png");

    /** The Constant PREVIOUS_TRAY. */
    public static final ImageIcon PREVIOUS_TRAY = SystemProperties.OS.isVista() ? getImage("previousTrayVista.png") : getImage("previousTray.png");

    /** The Constant PREVIOUS_TRAY_MENU. */
    public static final ImageIcon PREVIOUS_TRAY_MENU = getImage("previousTrayMenu.png");

    /** The Constant RADIO. */
    public static final ImageIcon RADIO = getImage("radio.png");

    /** The Constant RADIO_BIG. */
    public static final ImageIcon RADIO_BIG = getImage("radio_big.png");

    /** The Constant RADIO_FAVORITE. */
    public static final ImageIcon RADIO_FAVORITE = getImage("radio_favorite.png");

    /** The Constant RADIO_LITTLE. */
    public static final ImageIcon RADIO_LITTLE = getImage("radio_little.png");

    /** The Constant REFRESH. */
    public static final ImageIcon REFRESH = getImage("refresh.png");

    /** The Constant REMOVE. */
    public static final ImageIcon REMOVE = getImage("remove.png");

    /** The Constant REPEAT. */
    public static final ImageIcon REPEAT = getImage("repeat.png");

    /** The Constant REPORT_BUG_OR_REQUEST_FEATURE. */
    public static final ImageIcon REPORT_BUG_OR_REQUEST_FEATURE = getImage("reportBugOrRequestFeature.png");

    /** The Constant RSS_ADD. */
    public static final ImageIcon RSS_ADD = getImage("rss-add.png");

    /** The Constant RSS_LITTLE. */
    public static final ImageIcon RSS_LITTLE = getImage("rss_little.png");

    /** The Constant RSS. */
    public static final ImageIcon RSS = getImage("rss.png");

    /** The Constant RSS_BIG. */
    public static final ImageIcon RSS_BIG = getImage("rss_big.png");

    /** The Constant TAG. */
    public static final ImageIcon TAG = getImage("tag.png");

    /** The Constant SAVE. */
    public static final ImageIcon SAVE = getImage("save.png");

    /** The Constant SCROLL_PLAYLIST. */
    public static final ImageIcon SCROLL_PLAYLIST = getImage("scrollPlayList.png");

    /** The Constant SEARCH. */
    public static final ImageIcon SEARCH = getImage("search.png");

    /** The Constant SEARCH_ARTIST. */
    public static final ImageIcon SEARCH_ARTIST = getImage("searchArtist.png");

    /** The Constant SEARCH_AT. */
    public static final ImageIcon SEARCH_AT = getImage("searchWith.png");

    /** The Constant SHOW_LOG_FILE. */
    public static final ImageIcon SHOW_LOG_FILE = getImage("showLogFile.png");

    /** The Constant SHUFFLE. */
    public static final ImageIcon SHUFFLE = getImage("shuffle.png");

    /** The Constant SHUFFLE_PLAYLIST. */
    public static final ImageIcon SHUFFLE_PLAYLIST = getImage("shufflePlaylist.png");

    /** The Constant STATS. */
    public static final ImageIcon STATS = getImage("stats.png");

    /** The Constant STATUS_BAR. */
    public static final ImageIcon STATUS_BAR = getImage("statusBar.png");

    /** The Constant STOP. */
    public static final ImageIcon STOP = getImage("stop.png");

    /** The Constant STOP_TINY. */
    public static final ImageIcon STOP_TINY = getImage("stopTiny.png");

    /** The Constant STOP_TRAY. */
    public static final ImageIcon STOP_TRAY = SystemProperties.OS.isVista() ? getImage("stopTrayVista.png") : getImage("stopTray.png");

    /** The Constant STOP_TRAY_MENU. */
    public static final ImageIcon STOP_TRAY_MENU = getImage("stopTrayMenu.png");

    /** The Constant TITLE. */
    public static final ImageIcon TITLE = getImage("tag_title.png");

    /** The Constant TOOL_BAR. */
    public static final ImageIcon TOOL_BAR = getImage("toolBar.png");

    /** The Constant UNDO. */
    public static final ImageIcon UNDO = getImage("undo.png");

    /** The Constant VOLUME. */
    public static final ImageIcon VOLUME = getImage("volume-tiny.png");

    /** The Constant VOlUME_MAX. */
    public static final ImageIcon VOlUME_MAX = getImage("volume-max.png");

    /** The Constant VOLUME_MED. */
    public static final ImageIcon VOLUME_MED = getImage("volume-med.png");

    /** The Constant VOLUME_MIN. */
    public static final ImageIcon VOLUME_MIN = getImage("volume-min.png");

    /** The Constant VOLUME_MUTE. */
    public static final ImageIcon VOLUME_MUTE = getImage("volume-mute.png");

    /** The Constant VOLUME_MUTE_TRAY_MENU. */
    public static final ImageIcon VOLUME_MUTE_TRAY_MENU = getImage("volume-mute-tray-menu.png");

    /** The Constant VOLUME_ZERO. */
    public static final ImageIcon VOLUME_ZERO = getImage("volume-zero.png");

    /** The Constant ONE_STAR. */
    public static final ImageIcon ONE_STAR = getImage("1_star.png");

    /** The Constant TWO_STAR. */
    public static final ImageIcon TWO_STAR = getImage("2_star.png");

    /** The Constant THREE_STAR. */
    public static final ImageIcon THREE_STAR = getImage("3_star.png");

    /** The Constant FOUR_STAR. */
    public static final ImageIcon FOUR_STAR = getImage("4_star.png");

    /** The Constant FIVE_STAR. */
    public static final ImageIcon FIVE_STAR = getImage("5_star.png");

    /** The Constant YOUTUBE. */
    public static final ImageIcon YOUTUBE = getImage("Youtube_16x16.png");

    /**
     * Returns an image.
     * 
     * @param imgName
     *            the img name
     * 
     * @return An ImageIcon
     */
    private static ImageIcon getImage(String imgName) {
        URL imgURL = ImageLoader.class.getResource(imgName);
        return imgURL != null ? new ImageIcon(imgURL) : null;
    }
}
