/*
 * aTunes 1.13.6
 * Copyright (C) 2006-2009 Alex Aranda, Sylvain Gaudard, Thomas Beckers and contributors
 *
 * See http://www.atunes.org/wiki/index.php?title=Contributing for information about contributors
 *
 * http://www.atunes.org
 * http://sourceforge.net/projects/atunes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package net.sourceforge.atunes.gui.model;

import java.util.ArrayList;
import java.util.List;

import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.TableModel;

import net.sourceforge.atunes.gui.views.controls.playList.PlayListColumns;
import net.sourceforge.atunes.gui.views.controls.playList.PlayListColumns.PlayListColumn;
import net.sourceforge.atunes.kernel.modules.playlist.PlayListHandler;
import net.sourceforge.atunes.model.AudioObject;
import net.sourceforge.atunes.utils.LanguageTool;

/**
 * The Class PlayListTableModel.
 * 
 * @author fleax
 */
public class PlayListTableModel implements TableModel {

    /** The listeners. */
    private List<TableModelListener> listeners;

    /**
     * Constructor.
     * 
     * @param table
     *            the table
     */
    public PlayListTableModel() {
        listeners = new ArrayList<TableModelListener>();
    }

    /**
     * Adds a listener.
     * 
     * @param l
     *            the l
     */
    @Override
    public void addTableModelListener(TableModelListener l) {
        listeners.add(l);
    }

    /**
     * Returns column data class.
     * 
     * @param colIndex
     *            the col index
     * 
     * @return the column class
     */
    @Override
    public Class<?> getColumnClass(int colIndex) {
        return PlayListColumns.getColumns().get(PlayListColumns.getColumnId(colIndex)).getColumnClass();
    }

    /**
     * Return column count.
     * 
     * @return the column count
     */
    @Override
    public int getColumnCount() {
        return PlayListColumns.getVisibleColumnCount();
    }

    /**
     * Return column name.
     * 
     * @param colIndex
     *            the col index
     * 
     * @return the column name
     */
    @Override
    public String getColumnName(int colIndex) {
        return LanguageTool.getString(PlayListColumns.getColumns().get(PlayListColumns.getColumnId(colIndex)).getHeaderText());
    }

    /**
     * Return row count.
     * 
     * @return the row count
     */
    @Override
    public int getRowCount() {
        return PlayListHandler.getInstance().getCurrentPlayList(true).size();
    }

    /**
     * Returns value of a row and column.
     * 
     * @param rowIndex
     *            the row index
     * @param colIndex
     *            the col index
     * 
     * @return the value at
     */
    @Override
    public Object getValueAt(int rowIndex, int colIndex) {
        // Call Column method to get value from AudioFile
        return PlayListColumns.getColumns().get(PlayListColumns.getColumnId(colIndex)).getValueFor(PlayListHandler.getInstance().getCurrentPlayList(true).get(rowIndex));
    }

    /**
     * Returns if a cell is editable.
     * 
     * @param rowIndex
     *            the row index
     * @param columnIndex
     *            the column index
     * 
     * @return true, if checks if is cell editable
     */
    @Override
    public boolean isCellEditable(int rowIndex, int columnIndex) {
        // Column ID
        PlayListColumn c = PlayListColumns.getColumnId(columnIndex);

        // Call Column method to see if is editable
        return PlayListColumns.getColumns().get(c).isEditable();
    }

    /**
     * Refresh table.
     */
    public void refresh() {
        TableModelEvent event;
        event = new TableModelEvent(this, -1, -1, TableModelEvent.ALL_COLUMNS, TableModelEvent.UPDATE);

        for (int i = 0; i < listeners.size(); i++) {
            listeners.get(i).tableChanged(event);
        }
    }

    /**
     * Removes a listener.
     * 
     * @param l
     *            the l
     */
    @Override
    public void removeTableModelListener(TableModelListener l) {
        listeners.remove(l);
    }

    /**
     * Sets value for a cell. Does nothing as cells are not editable
     * 
     * @param aValue
     *            the a value
     * @param rowIndex
     *            the row index
     * @param columnIndex
     *            the column index
     */
    @Override
    public void setValueAt(Object aValue, int rowIndex, int columnIndex) {
        // AudioFile
        AudioObject file = PlayListHandler.getInstance().getCurrentPlayList(true).get(rowIndex);

        // Column ID
        PlayListColumn c = PlayListColumns.getColumnId(columnIndex);

        // Call column set value
        PlayListColumns.getColumns().get(c).setValueFor(file, aValue);
    }
}
