/*
 * aTunes 1.13.6
 * Copyright (C) 2006-2009 Alex Aranda, Sylvain Gaudard, Thomas Beckers and contributors
 *
 * See http://www.atunes.org/wiki/index.php?title=Contributing for information about contributors
 *
 * http://www.atunes.org
 * http://sourceforge.net/projects/atunes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package net.sourceforge.atunes.kernel.actions;

import java.awt.event.ActionEvent;
import java.util.ArrayList;
import java.util.List;

import javax.swing.AbstractAction;

import net.sourceforge.atunes.gui.views.controls.playList.PlayListTable;
import net.sourceforge.atunes.gui.views.dialogs.PatternInputDialog;
import net.sourceforge.atunes.kernel.modules.pattern.Pattern;
import net.sourceforge.atunes.kernel.modules.playlist.PlayListHandler;
import net.sourceforge.atunes.kernel.modules.repository.audio.AudioFile;
import net.sourceforge.atunes.kernel.modules.repository.tags.writer.EditTagFromFileNamePatternProcess;
import net.sourceforge.atunes.kernel.modules.visual.VisualHandler;
import net.sourceforge.atunes.model.AudioObject;
import net.sourceforge.atunes.utils.LanguageTool;
import net.sourceforge.atunes.utils.StringUtils;

/**
 * Calls process to set tag from a pattern of file name entered by user
 * 
 * @author fleax
 * 
 */
public class AutoSetTagFromFileNamePatternAction extends AbstractAction {

    private static final long serialVersionUID = -8458591967408812850L;

    public AutoSetTagFromFileNamePatternAction() {
        super(StringUtils.getString(LanguageTool.getString("AUTO_SET_TAG_FROM_FILE_NAME_PATTERN"), "..."));
        putValue(SHORT_DESCRIPTION, LanguageTool.getString("AUTO_SET_TAG_FROM_FILE_NAME_PATTERN"));
    }

    @Override
    public void actionPerformed(ActionEvent e) {
        // Get selected AudioFiles in play list
        PlayListTable table = VisualHandler.getInstance().getPlayListTable();
        int[] rows = table.getSelectedRows();
        List<AudioFile> files = new ArrayList<AudioFile>();
        for (int element : rows) {
            AudioObject ao = PlayListHandler.getInstance().getCurrentPlayList(true).get(element);
            if (ao instanceof AudioFile) {
                files.add((AudioFile) ao);
            }
        }

        if (!files.isEmpty()) {
            // Show pattern input dialog
            PatternInputDialog inputDialog = new PatternInputDialog(VisualHandler.getInstance().getFrame().getFrame(), false);
            inputDialog.show(Pattern.getRecognitionPatterns(), files.get(0).getNameWithoutExtension());
            String pattern = inputDialog.getResult();

            // If user entered a pattern apply to files
            if (pattern != null) {
                EditTagFromFileNamePatternProcess process = new EditTagFromFileNamePatternProcess(files, pattern);
                process.execute();
            }
        }
    }

}
