/*
 * aTunes 1.13.6
 * Copyright (C) 2006-2009 Alex Aranda, Sylvain Gaudard, Thomas Beckers and contributors
 *
 * See http://www.atunes.org/wiki/index.php?title=Contributing for information about contributors
 *
 * http://www.atunes.org
 * http://sourceforge.net/projects/atunes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package net.sourceforge.atunes.kernel.actions;

import java.awt.event.ActionEvent;
import java.util.List;

import javax.swing.AbstractAction;
import javax.swing.SwingUtilities;

import net.sourceforge.atunes.gui.images.ImageLoader;
import net.sourceforge.atunes.gui.model.NavigationTableModel;
import net.sourceforge.atunes.kernel.ControllerProxy;
import net.sourceforge.atunes.kernel.modules.device.TransferToRepositoryProcess;
import net.sourceforge.atunes.kernel.modules.process.ProcessListener;
import net.sourceforge.atunes.kernel.modules.repository.RepositoryHandler;
import net.sourceforge.atunes.kernel.modules.repository.audio.AudioFile;
import net.sourceforge.atunes.kernel.modules.visual.VisualHandler;
import net.sourceforge.atunes.model.AudioObject;
import net.sourceforge.atunes.utils.LanguageTool;

public class CopyToRepositoryAction extends AbstractAction {

    private static final long serialVersionUID = 2416674807979541242L;

    CopyToRepositoryAction() {
        super(LanguageTool.getString("COPY_TO_REPOSITORY"), ImageLoader.EXPORT);
        putValue(SHORT_DESCRIPTION, LanguageTool.getString("COPY_TO_REPOSITORY"));
    }

    @Override
    public void actionPerformed(ActionEvent e) {
        List<AudioObject> songs = ((NavigationTableModel) ControllerProxy.getInstance().getNavigationController().getNavigationPanel().getNavigationTable().getModel()).getSongs();
        final TransferToRepositoryProcess importer = new TransferToRepositoryProcess(AudioFile.getAudioFiles(songs));
        importer.addProcessListener(new ProcessListener() {
            @Override
            public void processCanceled() { /* Nothing to do */
            }

            @Override
            public void processFinished(final boolean ok) {
                SwingUtilities.invokeLater(new Runnable() {
                    @Override
                    public void run() {
                        if (!ok) {
                            VisualHandler.getInstance().showErrorDialog(LanguageTool.getString("ERRORS_IN_COPYING_PROCESS"));
                        }
                        // Force a refresh of repository to add new songs
                        RepositoryHandler.getInstance().refreshRepository();
                    }
                });
            }
        });
        importer.execute();
    }

}
