/*
 * aTunes 1.13.6
 * Copyright (C) 2006-2009 Alex Aranda, Sylvain Gaudard, Thomas Beckers and contributors
 *
 * See http://www.atunes.org/wiki/index.php?title=Contributing for information about contributors
 *
 * http://www.atunes.org
 * http://sourceforge.net/projects/atunes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package net.sourceforge.atunes.kernel.modules.navigator;

import java.awt.Component;
import java.awt.event.MouseEvent;
import java.util.ArrayList;
import java.util.Collections;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import javax.swing.ImageIcon;
import javax.swing.JLabel;
import javax.swing.JMenuItem;
import javax.swing.JPopupMenu;
import javax.swing.JSeparator;
import javax.swing.JTable;
import javax.swing.JTree;
import javax.swing.tree.DefaultMutableTreeNode;
import javax.swing.tree.DefaultTreeModel;
import javax.swing.tree.TreeCellRenderer;
import javax.swing.tree.TreeNode;

import net.sourceforge.atunes.gui.ColorDefinitions;
import net.sourceforge.atunes.gui.images.ImageLoader;
import net.sourceforge.atunes.gui.model.NavigationTableModel;
import net.sourceforge.atunes.gui.model.NavigationTableModel.Property;
import net.sourceforge.atunes.gui.views.controls.DragSourceTree;
import net.sourceforge.atunes.kernel.actions.Actions;
import net.sourceforge.atunes.kernel.actions.AddToPlayListAction;
import net.sourceforge.atunes.kernel.actions.ClearTagAction;
import net.sourceforge.atunes.kernel.actions.CopyToRepositoryAction;
import net.sourceforge.atunes.kernel.actions.EditTagFromNavigatorAction;
import net.sourceforge.atunes.kernel.actions.ExtractPictureAction;
import net.sourceforge.atunes.kernel.actions.FillDeviceWithRandomSongsAction;
import net.sourceforge.atunes.kernel.actions.FilterNavigatorAction;
import net.sourceforge.atunes.kernel.actions.MarkPodcastEntryListenedAction;
import net.sourceforge.atunes.kernel.actions.OpenFolderFromNavigatorAction;
import net.sourceforge.atunes.kernel.actions.PlayNowAction;
import net.sourceforge.atunes.kernel.actions.RemoveFromDiskAction;
import net.sourceforge.atunes.kernel.actions.SetAsPlayListAction;
import net.sourceforge.atunes.kernel.controllers.navigation.NavigationController.ViewMode;
import net.sourceforge.atunes.kernel.modules.device.DeviceHandler;
import net.sourceforge.atunes.kernel.modules.favorites.FavoritesHandler;
import net.sourceforge.atunes.kernel.modules.repository.audio.AudioFile;
import net.sourceforge.atunes.kernel.modules.repository.model.Album;
import net.sourceforge.atunes.kernel.modules.repository.model.Artist;
import net.sourceforge.atunes.kernel.modules.repository.model.Folder;
import net.sourceforge.atunes.kernel.modules.repository.model.Genre;
import net.sourceforge.atunes.kernel.modules.state.ApplicationState;
import net.sourceforge.atunes.model.AudioObject;
import net.sourceforge.atunes.model.TreeObject;
import net.sourceforge.atunes.utils.LanguageTool;
import net.sourceforge.atunes.utils.StringUtils;

import org.jvnet.substance.api.renderers.SubstanceDefaultTreeCellRenderer;

public class DeviceNavigationView extends NavigationView {

    /** The device tree. */
    private JTree deviceTree;

    /** The device tree menu. */
    private JPopupMenu deviceTreeMenu;

    /** The device table menu. */
    private JPopupMenu deviceTableMenu;

    @Override
    public ImageIcon getIcon() {
        return ImageLoader.DEVICE;
    }

    @Override
    public String getTitle() {
        return LanguageTool.getString("DEVICE");
    }

    @Override
    public String getTooltip() {
        return LanguageTool.getString("DEVICE_TAB_TOOLTIP");
    }

    @Override
    public JTree getTree() {
        if (deviceTree == null) {
            deviceTree = new DragSourceTree(new DefaultTreeModel(new DefaultMutableTreeNode(LanguageTool.getString("DEVICE"))));
            deviceTree.setToggleClickCount(0);
            deviceTree.setCellRenderer(getTreeRenderer());
        }
        return deviceTree;
    }

    @Override
    public boolean isAudioObjectsFromNodeNeedSort() {
        return true;
    }

    @Override
    public JPopupMenu getTreePopupMenu() {
        if (deviceTreeMenu == null) {
            deviceTreeMenu = new JPopupMenu();
            deviceTreeMenu.add(new JMenuItem(Actions.getAction(AddToPlayListAction.class)));
            deviceTreeMenu.add(new JMenuItem(Actions.getAction(SetAsPlayListAction.class)));
            deviceTreeMenu.add(new JSeparator());
            deviceTreeMenu.add(new JMenuItem(Actions.getAction(EditTagFromNavigatorAction.class)));
            deviceTreeMenu.add(new JMenuItem(Actions.getAction(ClearTagAction.class)));
            deviceTreeMenu.add(new JSeparator());
            deviceTreeMenu.add(new JMenuItem(Actions.getAction(RemoveFromDiskAction.class)));
            deviceTreeMenu.add(new JSeparator());
            deviceTreeMenu.add(new JMenuItem(Actions.getAction(CopyToRepositoryAction.class)));
            deviceTreeMenu.add(new JMenuItem(Actions.getAction(FillDeviceWithRandomSongsAction.class)));
            deviceTreeMenu.add(new JSeparator());
            deviceTreeMenu.add(new JMenuItem(Actions.getAction(FilterNavigatorAction.class)));
        }
        return deviceTreeMenu;
    }

    @Override
    public JPopupMenu getTablePopupMenu() {
        if (deviceTableMenu == null) {
            deviceTableMenu = new JPopupMenu();
            deviceTableMenu.add(new JMenuItem(Actions.getAction(AddToPlayListAction.class)));
            deviceTableMenu.add(new JMenuItem(Actions.getAction(SetAsPlayListAction.class)));
            deviceTableMenu.add(new JMenuItem(Actions.getAction(PlayNowAction.class)));
            deviceTableMenu.add(new JSeparator());
            deviceTableMenu.add(new JMenuItem(Actions.getAction(OpenFolderFromNavigatorAction.class)));
            deviceTableMenu.add(new JSeparator());
            deviceTableMenu.add(new JMenuItem(Actions.getAction(EditTagFromNavigatorAction.class)));
            deviceTableMenu.add(new JMenuItem(Actions.getAction(ClearTagAction.class)));
            deviceTableMenu.add(new JMenuItem(Actions.getAction(ExtractPictureAction.class)));
            deviceTableMenu.add(new JSeparator());
            deviceTableMenu.add(new JMenuItem(Actions.getAction(RemoveFromDiskAction.class)));
            deviceTableMenu.add(new JSeparator());
            deviceTableMenu.add(new JMenuItem(Actions.getAction(CopyToRepositoryAction.class)));
            deviceTableMenu.add(new JSeparator());
            deviceTableMenu.add(new JMenuItem(Actions.getAction(FilterNavigatorAction.class)));
        }
        return deviceTableMenu;
    }

    @Override
    protected Map<String, ?> getViewData(ViewMode viewMode) {
        Map<String, ?> data;
        if (viewMode == ViewMode.GENRE) {
            data = DeviceHandler.getInstance().getDeviceGenreStructure();
        } else if (viewMode == ViewMode.FOLDER) {
            data = DeviceHandler.getInstance().getDeviceFolderStructure();
        } else {
            data = DeviceHandler.getInstance().getDeviceArtistAndAlbumStructure();
        }
        return data;
    }

    @Override
    protected void refreshTree(ViewMode viewMode, String treeFilter) {
        debug("Refreshing " + this.getClass().getName());

        Map<String, ?> data = getViewData(viewMode);

        DefaultTreeModel treeModel = (DefaultTreeModel) getTree().getModel();
        DefaultMutableTreeNode root = (DefaultMutableTreeNode) treeModel.getRoot();

        // Nodes to be selected after refresh
        List<DefaultMutableTreeNode> nodesToSelect = new ArrayList<DefaultMutableTreeNode>();
        // Nodes to be expanded after refresh
        List<DefaultMutableTreeNode> nodesToExpand = new ArrayList<DefaultMutableTreeNode>();

        // Get objects selected before refreshing tree
        List<TreeObject> objectsSelected = RefreshUtils.getTreeObjectsSelected(getTree());
        // Get objects expanded before refreshing tree
        List<TreeObject> objectsExpanded = RefreshUtils.getTreeObjectsExpanded(getTree(), root);

        root.removeAllChildren();
        List<String> artistNamesList = new ArrayList<String>(data.keySet());
        if (ApplicationState.getInstance().isUseSmartTagViewSorting()) {
            Collections.sort(artistNamesList, smartComparator);
        } else {
            Collections.sort(artistNamesList, comparator);
        }

        if (viewMode == ViewMode.ARTIST) {
            for (int i = 0; i < artistNamesList.size(); i++) {
                Artist artist = (Artist) data.get(artistNamesList.get(i));
                DefaultMutableTreeNode artistNode = new DefaultMutableTreeNode(artist);
                List<String> albumNamesList = new ArrayList<String>(artist.getAlbums().keySet());
                if (ApplicationState.getInstance().isUseSmartTagViewSorting()) {
                    Collections.sort(albumNamesList, smartComparator);
                } else {
                    Collections.sort(albumNamesList, comparator);
                }
                if (treeFilter == null || artist.getName().toUpperCase().contains(treeFilter.toUpperCase())) {
                    for (int j = 0; j < albumNamesList.size(); j++) {
                        Album album = artist.getAlbum(albumNamesList.get(j));
                        DefaultMutableTreeNode albumNode = new DefaultMutableTreeNode(album);
                        // If node was selected before refreshing...
                        if (objectsSelected.contains(albumNode.getUserObject())) {
                            nodesToSelect.add(albumNode);
                        }
                        artistNode.add(albumNode);
                    }
                    root.add(artistNode);
                    //	Reload causes very important lag on large collections and if it is not used
                    // selection does not work.

                    // If node was selected before refreshing...
                    if (objectsSelected.contains(artistNode.getUserObject())) {
                        nodesToSelect.add(artistNode);
                    }
                    // If node was expanded before refreshing...
                    if (objectsExpanded.contains(artistNode.getUserObject())) {
                        nodesToExpand.add(artistNode);
                    }
                }
            }
        } else if (viewMode == ViewMode.ALBUM) { // Album view
            // Gather albums
            Map<String, Album> albums = new HashMap<String, Album>();
            DefaultMutableTreeNode albumNode = null;
            for (int i = 0; i < artistNamesList.size(); i++) {
                Artist artist = (Artist) data.get(artistNamesList.get(i));
                List<String> albumNamesList = new ArrayList<String>(artist.getAlbums().keySet());
                Collections.sort(albumNamesList);
                for (int j = 0; j < albumNamesList.size(); j++) {
                    String a = albumNamesList.get(j);
                    Album album = artist.getAlbum(a);
                    if (treeFilter == null || album.getName().toUpperCase().contains(treeFilter.toUpperCase())) {
                        albums.put(album.getName() + artist.toString(), album);
                    }
                }
            }

            // Then add them in the tree with the artists
            List<String> albumsList = new ArrayList<String>(albums.keySet());
            Collections.sort(albumsList, comparator);
            for (int i = 0; i < albumsList.size(); i++) {
                Album a = albums.get(albumsList.get(i));
                DefaultMutableTreeNode artistNode = new DefaultMutableTreeNode(a.getArtist());
                if (i == 0 || !albums.get(albumsList.get(i)).toString().equals((albums.get(albumsList.get(i - 1))).toString())) {
                    albumNode = new DefaultMutableTreeNode(a);
                    root.add(albumNode);
                    // If node was selected before refreshing...
                    if (objectsSelected.contains(albumNode.getUserObject())) {
                        nodesToSelect.add(albumNode);
                    }
                }
                if (albumNode != null) {
                    albumNode.add(artistNode);
                    // That one is a workaround because the Artist class returns all audiofiles of an artist and not only 
                    // the songs from a specific album, so we add that album again. If you have a better idea you can improve
                    // DefaultMutableTreeNode subAlbumNode = new DefaultMutableTreeNode(a.getArtist().getAlbum(a.getName()));
                    // artistNode.add(subAlbumNode);
                }
            }
        } else if (viewMode == ViewMode.GENRE) { // Genre view
            List<String> genreNamesList = new ArrayList<String>(data.keySet());
            Collections.sort(genreNamesList, comparator);

            for (int i = 0; i < genreNamesList.size(); i++) {
                Genre genre = (Genre) data.get(genreNamesList.get(i));
                if (treeFilter == null || genre.getName().toUpperCase().contains(treeFilter.toUpperCase())) {
                    DefaultMutableTreeNode genreNode = new DefaultMutableTreeNode(genre);
                    // If node was selected before refreshing...
                    if (objectsSelected.contains(genreNode.getUserObject())) {
                        nodesToSelect.add(genreNode);
                    }
                    // If node was expanded before refreshing...
                    if (objectsExpanded.contains(genreNode.getUserObject())) {
                        nodesToExpand.add(genreNode);
                    }
                    artistNamesList = new ArrayList<String>(genre.getArtists().keySet());
                    Collections.sort(artistNamesList);
                    for (int j = 0; j < artistNamesList.size(); j++) {
                        Artist artist = genre.getArtist(artistNamesList.get(j));
                        DefaultMutableTreeNode artistNode = new DefaultMutableTreeNode(artist);
                        List<String> albumNamesList = new ArrayList<String>(artist.getAlbums().keySet());
                        Collections.sort(albumNamesList);
                        for (int k = 0; k < albumNamesList.size(); k++) {
                            Album album = artist.getAlbum(albumNamesList.get(k));
                            DefaultMutableTreeNode albumNode = new DefaultMutableTreeNode(album);
                            artistNode.add(albumNode);
                            genreNode.add(artistNode);
                            // If node was selected before refreshing...
                            if (objectsSelected.contains(artistNode.getUserObject())) {
                                nodesToSelect.add(artistNode);
                            }
                            // If node was selected before refreshing...
                            if (objectsSelected.contains(albumNode.getUserObject())) {
                                nodesToSelect.add(albumNode);
                            }
                            // If node was expanded before refreshing...
                            if (objectsExpanded.contains(artistNode.getUserObject()) && objectsExpanded.contains(((DefaultMutableTreeNode) artistNode.getParent()).getUserObject())) {
                                nodesToExpand.add(artistNode);
                            }
                        }
                    }
                    root.add(genreNode);
                }
            }
        } else { // Folder view
            List<String> rootFolderKeys = new ArrayList<String>(data.keySet());
            if (rootFolderKeys.isEmpty()) {
                root.setUserObject(LanguageTool.getString("DEVICE"));
            } else {
                root.setUserObject(StringUtils.getString(LanguageTool.getString("DEVICE"), " (", rootFolderKeys.get(0), ")"));
                Folder rootFolder = (Folder) data.get(rootFolderKeys.get(0));
                RefreshUtils.addFolderNodes(rootFolder.getFolders(), root, null, comparator);
            }
        }
        // Reload the tree to refresh content
        treeModel.reload();

        if (viewMode == ViewMode.FOLDER) {
            if (objectsExpanded.isEmpty()) {
                // In folder view root child nodes must be expanded always
                // So when refreshing folder view for first time add these nodes to list of expanded objects
                for (int i = 0; i < root.getChildCount(); i++) {
                    TreeNode childNode = root.getChildAt(i);
                    objectsExpanded.add((TreeObject) ((DefaultMutableTreeNode) childNode).getUserObject());
                }
            }

            // Get nodes to select after refresh
            nodesToSelect = RefreshUtils.getNodes(root, objectsSelected);

            // Get nodes to expand after refresh
            nodesToExpand = RefreshUtils.getNodes(root, objectsExpanded);
        }

        // Expand nodes
        RefreshUtils.expandNodes(getTree(), nodesToExpand);

        // Once tree has been refreshed, select previously selected nodes
        RefreshUtils.selectNodes(getTree(), nodesToSelect);
    }

    @Override
    public List<AudioObject> getAudioObjectForTreeNode(DefaultMutableTreeNode node, ViewMode viewMode, String treeFilter) {
        List<AudioObject> songs = new ArrayList<AudioObject>();
        if (node.isRoot()) {
            if (treeFilter == null) {
                songs.addAll(DeviceHandler.getInstance().getAudioFilesList());
            } else {
                songs = new ArrayList<AudioObject>();
                for (int i = 0; i < node.getChildCount(); i++) {
                    TreeObject obj = (TreeObject) ((DefaultMutableTreeNode) node.getChildAt(i)).getUserObject();
                    songs.addAll(obj.getAudioObjects());
                }
            }
        } else {
            TreeObject obj = (TreeObject) node.getUserObject();
            songs = obj.getAudioObjects();
        }
        return songs;
    }

    @Override
    public Class<?> getNavigatorTableColumnClass(int columnIndex) {
        switch (columnIndex) {
        case 0:
            return Property.class;
        case 1:
            return String.class;
        case 2:
            return Long.class;
        default:
            return Long.class;
        }
    }

    @Override
    public int getNavigatorTableColumnCount() {
        return 3;
    }

    @Override
    public String getNavigatorTableColumnName(int columnIndex) {
        switch (columnIndex) {
        case 0:
            return "";
        case 1:
            return LanguageTool.getString("FILE");
        case 2:
            return LanguageTool.getString("DURATION");
        default:
            return "";
        }
    }

    @Override
    public Object getNavigatorTableValueAt(AudioObject audioObject, int columnIndex) {
        if (audioObject instanceof AudioFile) {
            if (columnIndex == 0) {
                return ApplicationState.getInstance().isShowFavoritesInNavigator() && FavoritesHandler.getInstance().getFavoriteSongs().contains(audioObject) ? Property.FAVORITE
                        : Property.NO_PROPERTIES;
            } else if (columnIndex == 1) {
                if (ApplicationState.getInstance().getViewMode() != ViewMode.FOLDER) {
                    return audioObject.getTitleOrFileName();
                }
                return ((AudioFile) audioObject).getFile().getName();
            } else if (columnIndex == 2) {
                return audioObject.getDuration();
            }
        }
        return "";
    }

    @Override
    public int getNavigatorTableMaxWidthForColumn(int columnIndex) {
        if (columnIndex == 0) {
            return 20;
        } else if (columnIndex == 2) {
            return 50;
        }
        return -1;
    }

    @Override
    public boolean isViewModeSupported() {
        return true;
    }

    @Override
    public List<AudioObject> filterNavigatorTable(List<AudioObject> audioObjects, String filter) {
        // Filter by title
        List<AudioObject> filteredObjects = new ArrayList<AudioObject>();
        for (AudioObject audioObject : audioObjects) {
            if (audioObject.getTitleOrFileName().toLowerCase().contains(filter.toLowerCase())) {
                filteredObjects.add(audioObject);
            }
        }
        return filteredObjects;
    }

    @Override
    public boolean isNavigatorTableFilterSupported() {
        return true;
    }

    @Override
    public void updateTreePopupMenuWithTreeSelection(MouseEvent e) {
        Actions.getAction(MarkPodcastEntryListenedAction.class).setEnabled(false);
        Actions.getAction(PlayNowAction.class).setEnabled(false);
    }

    @Override
    public void updateTablePopupMenuWithTableSelection(JTable table, MouseEvent e) {
        Actions.getAction(PlayNowAction.class).setEnabled(table.getSelectedRowCount() == 1);
        AudioObject ao = ((NavigationTableModel) table.getModel()).getSongAt(table.getSelectedRow());
        Actions.getAction(ExtractPictureAction.class).setEnabled(table.getSelectedRowCount() == 1 && ao instanceof AudioFile && ((AudioFile) ao).hasInternalPicture());
    }

    @Override
    protected TreeCellRenderer getTreeRenderer() {
        return new SubstanceDefaultTreeCellRenderer() {
            private static final long serialVersionUID = -7992021225213275134L;

            @Override
            public Component getTreeCellRendererComponent(JTree tree, Object value, boolean selected, boolean expanded, boolean leaf, int row, boolean hasFocus) {
                DefaultMutableTreeNode node = (DefaultMutableTreeNode) value;
                final Object content = node.getUserObject();

                JLabel label = (JLabel) super.getTreeCellRendererComponent(tree, value, selected, expanded, leaf, row, hasFocus);
                if (content instanceof Artist) {
                    label.setIcon(ImageLoader.ARTIST);
                } else if (content instanceof Album) {
                    label.setIcon(ImageLoader.ALBUM);
                } else if (content instanceof Genre) {
                    label.setIcon(ImageLoader.GENRE);
                } else if (content instanceof Folder) {
                    label.setIcon(ImageLoader.FOLDER);
                } else {
                    label.setIcon(ImageLoader.DEVICE);
                }

                if (value.toString() != null) {
                    if (value.toString().equals(LanguageTool.getString("UNKNOWN_ARTIST")) || value.toString().equals(LanguageTool.getString("UNKNOWN_ALBUM"))
                            || value.toString().equals(LanguageTool.getString("UNKNOWN_GENRE"))) {
                        label.setForeground(ColorDefinitions.GENERAL_UNKNOWN_ELEMENT_FOREGROUND_COLOR);
                    }
                }
                return label;
            }
        };
    }
}
