/*
 * aTunes 1.13.6
 * Copyright (C) 2006-2009 Alex Aranda, Sylvain Gaudard, Thomas Beckers and contributors
 *
 * See http://www.atunes.org/wiki/index.php?title=Contributing for information about contributors
 *
 * http://www.atunes.org
 * http://sourceforge.net/projects/atunes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package net.sourceforge.atunes.kernel.modules.navigator;

import java.util.ArrayList;
import java.util.HashMap;
import java.util.List;
import java.util.Map;

import net.sourceforge.atunes.kernel.ControllerProxy;
import net.sourceforge.atunes.kernel.modules.state.ApplicationState;

public class NavigationViews {

    private static List<NavigationView> navigationViews;

    private static Map<Class<? extends NavigationView>, NavigationView> navigationViewsMap;

    public static List<NavigationView> getNavigationViews() {
        if (navigationViews == null) {
            navigationViews = new ArrayList<NavigationView>();
            // TODO: Dynamic load of navigation views, possibly from a config file
            navigationViews.add(new RepositoryNavigationView());
            navigationViews.add(new FavoritesNavigationView());
            navigationViews.add(new DeviceNavigationView());
            navigationViews.add(new RadioNavigationView());
            navigationViews.add(new PodcastNavigationView());
        }
        return navigationViews;
    }

    private static Map<Class<? extends NavigationView>, NavigationView> getNavigationViewsMap() {
        if (navigationViewsMap == null) {
            navigationViewsMap = new HashMap<Class<? extends NavigationView>, NavigationView>();
            for (NavigationView view : getNavigationViews()) {
                navigationViewsMap.put(view.getClass(), view);
            }
        }
        return navigationViewsMap;
    }

    public static NavigationView getCurrentView() {
        return NavigationViews.getView(NavigationViews.getViewByName(ApplicationState.getInstance().getNavigationView()));
    }

    public static NavigationView getView(Class<? extends NavigationView> navigationViewClass) {
        return getNavigationViewsMap().get(navigationViewClass);
    }

    public static void refreshViews() {
        for (NavigationView view : getNavigationViews()) {
            view.refreshView(ApplicationState.getInstance().getViewMode(), ControllerProxy.getInstance().getNavigationController().getNavigationPanel().getTreeFilterPanel()
                    .getFilter());
        }
    }

    public static void refreshViews(Class<? extends NavigationView>... navigationViewClasses) {
        for (Class<? extends NavigationView> navigationViewClass : navigationViewClasses) {
            getView(navigationViewClass).refreshView(ApplicationState.getInstance().getViewMode(),
                    ControllerProxy.getInstance().getNavigationController().getNavigationPanel().getTreeFilterPanel().getFilter());
        }
    }

    public static Class<? extends NavigationView> getViewByName(String className) {
        if (className == null) {
            return null;
        }
        try {
            @SuppressWarnings("unchecked")
            Class<? extends NavigationView> view = (Class<? extends NavigationView>) Class.forName(className);
            for (Class<? extends NavigationView> viewFromMap : getNavigationViewsMap().keySet()) {
                if (viewFromMap.equals(view)) {
                    return viewFromMap;
                }
            }
            return null;
        } catch (ClassNotFoundException e) {
            return null;
        }
    }

    public static int indexOf(Class<? extends NavigationView> view) {
        return getNavigationViews().indexOf(getNavigationViewsMap().get(view));
    }
}
