/*
 * aTunes 1.13.6
 * Copyright (C) 2006-2009 Alex Aranda, Sylvain Gaudard, Thomas Beckers and contributors
 *
 * See http://www.atunes.org/wiki/index.php?title=Contributing for information about contributors
 *
 * http://www.atunes.org
 * http://sourceforge.net/projects/atunes
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version 2
 * of the License, or (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 */

package net.sourceforge.atunes.kernel.modules.player.vlcplayer;

import java.io.BufferedReader;
import java.io.IOException;
import java.io.InputStreamReader;

import javax.swing.SwingUtilities;

import net.sourceforge.atunes.misc.log.LogCategories;
import net.sourceforge.atunes.misc.log.Logger;
import net.sourceforge.atunes.utils.ClosingUtils;

/**
 * The Class MPlayerErrorReader.
 */
class VlcOutputReader extends Thread {

    /**
     * The logger used in player engines
     */
    protected static Logger logger = new Logger();

    /** The handler. */
    VlcPlayerHandler handler;

    /** The in. */
    private BufferedReader in;

    /** */
    private String lastLineReaded = "";

    /** the sentence used by vlc to present radio stream infos */
    protected static final String ACCESS_HTTP_DEBUG = "access_http access debug: New Title=";

    /**
     * Instantiates a new m player error reader.
     * 
     * @param handler
     *            the handler
     * @param process
     *            the process
     * @param audioObject
     *            the audio object
     */
    VlcOutputReader(VlcPlayerHandler handler, Process process) {
        this.handler = handler;
        in = new BufferedReader(new InputStreamReader(process.getErrorStream()));
    }

    /**
     * Read.
     * 
     * @param line
     *            the line
     */
    protected void read(String line) {
        logger.debug(LogCategories.PLAYER, "VLC Debug Stream read : " + line);

        if (line.indexOf(ACCESS_HTTP_DEBUG) > 0) {
            //System.out.println("Stream info : " + line);
            this.lastLineReaded = line;
        }

    }

    /*
     * (non-Javadoc)
     * 
     * @see java.lang.Thread#run()
     */
    @Override
    public void run() {
        String line;
        try {
            while ((line = in.readLine()) != null && !isInterrupted()) {
                final String readed = line;
                SwingUtilities.invokeLater(new Runnable() {
                    public void run() {
                        read(readed);
                    }
                });
            }
        } catch (final IOException e) {
            SwingUtilities.invokeLater(new Runnable() {
                public void run() {
                    //PlayerHandler.notifyPlayerError(e);
                }
            });
        } finally {
            ClosingUtils.close(in);
        }
    }

    /**
     * @return lastLineReaded
     */
    public String getLastLineReaded() {
        return lastLineReaded;
    }

}
